<?php

class APPMAKER_WC_Options {
	/**
	 * Holds the values to be used in the fields callbacks
	 *
	 * @var object
	 */
	private $options;


	/**
	 * Start up
	 */
	public function __construct() {
		add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
		add_action( 'admin_init', array( $this, 'page_init' ) );
	}

	/**
	 * Add options page
	 */
	public function add_plugin_page() {
		// This page will be under "WooCommerce".
		add_submenu_page(
			'woocommerce',
			'Appmaker WooCommerce Mobile App Manager Settings',
			'Appmaker App Settings',
			'manage_options',
			'appmaker-wc-admin',
			array( $this, 'create_admin_page' )
		);
	}

	/**
	 * Options page callback
	 */
	public function create_admin_page() {
		// Set class property.
		$this->options = get_option( 'appmaker_wc_settings' );
		?>
        <div class="wrap">
            <h2>Appmaker Settings</h2>
			<?php if ( ! empty( $this->options['project_id'] ) ) {
				$auto_login      = false;
				$manage_url_base = "http://manage.appmaker.xyz";
				$manage_url      = $manage_url_base . '/apps/' . $this->options['project_id'] . '/';
				if ( $auto_login ) {
					$manage_url = site_url( '?rest_route=/appmaker-wc/v1/manage-login&url=' . $manage_url_base . '&return_to=' . "/apps/" . $this->options['project_id'] );
				}
				?>
                <div class="notice notice-success">
                    <p style="font-size: large">
                        <a <?php if ( ! $auto_login ) { ?> target="_blank" <?php } ?>
                                href="<?php echo $manage_url; ?>" class="">
							<?php echo __( 'Click here', 'appmaker-woocommerce-mobile-app-manager' ); ?> </a> <?php echo __( 'to Customize Navigation Menu, In-App Pages or send push notification.', 'appmaker-woocommerce-mobile-app-manager' ); ?>
                    </p>
                </div>
			<?php } ?>
            <form method="post" action="options.php">
				<?php
				// This prints out all hidden setting fields.
				settings_fields( 'appmaker_wc_key_options' );
				do_settings_sections( 'appmaker-wc-setting-admin' );
				submit_button();
				?>
            </form>
        </div>
		<?php
	}

	/**
	 * Register and add settings
	 */
	public function page_init() {


		register_setting(
			'appmaker_wc_key_options',
			'appmaker_wc_settings',
			array( $this, 'sanitize' )
		);

		add_settings_section(
			'appmaker_wc_setting_section',
			__( 'API key details', 'appmaker-woocommerce-mobile-app-manager' ),
			array( $this, 'print_section_info' ),
			'appmaker-wc-setting-admin'
		);

		add_settings_field(
			'project_id',
			__( 'Project ID', 'appmaker-woocommerce-mobile-app-manager' ),
			array( $this, 'project_id_callback' ),
			'appmaker-wc-setting-admin',
			'appmaker_wc_setting_section'
		);

		add_settings_field(
			'api_key',
			__( 'API Key', 'appmaker-woocommerce-mobile-app-manager' ),
			array( $this, 'api_key_callback' ),
			'appmaker-wc-setting-admin',
			'appmaker_wc_setting_section'
		);

		add_settings_field(
			'api_secret',
			__( 'API Secret', 'appmaker-woocommerce-mobile-app-manager' ),
			array( $this, 'api_secret_callback' ),
			'appmaker-wc-setting-admin',
			'appmaker_wc_setting_section'
		);

	}

	/**
	 * Sanitize each setting field as needed.
	 *
	 * @param array $input Contains all settings fields as array keys.
	 *
	 * @return array
	 */
	public function sanitize( $input ) {
		$new_input = array();
		if ( isset( $input['project_id'] ) ) {
			$new_input['project_id'] = sanitize_text_field( $input['project_id'] );
			if ( ! is_numeric( $new_input['project_id'] ) ) {
				$new_input['project_id'] = '';
			}
		}

		if ( isset( $input['api_key'] ) ) {
			$new_input['api_key'] = sanitize_text_field( $input['api_key'] );
		}

		if ( isset( $input['api_secret'] ) ) {
			$new_input['api_secret'] = sanitize_text_field( $input['api_secret'] );
		}


		return $new_input;
	}

	/**
	 * Print the Section text
	 */
	public function print_section_info() {
		print 'Enter your Appmaker API settings below:';
	}

	/**
	 * Get the settings option array and print one of its values
	 */
	public function project_id_callback() {
		printf(
			'<input type="text" id="project_id" name="appmaker_wc_settings[project_id]" value="%s" />',
			isset( $this->options['project_id'] ) ? esc_attr( $this->options['project_id'] ) : ''
		);
	}

	/**
	 * Get the settings option array and print one of its values
	 */
	public function api_key_callback() {
		printf(
			'<input type="text" id="api_key" name="appmaker_wc_settings[api_key]" value="%s" />',
			isset( $this->options['api_key'] ) ? esc_attr( $this->options['api_key'] ) : ''
		);
	}

	/**
	 * Get the settings option array and print one of its values
	 */
	public function api_secret_callback() {
		printf(
			'<input type="text" id="api_secret" name="appmaker_wc_settings[api_secret]" value="%s" />',
			isset( $this->options['api_secret'] ) ? esc_attr( $this->options['api_secret'] ) : ''
		);
	}
}

new APPMAKER_WC_Options();
