<?php
/**
 * REST API Cart controller
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * REST API Cart controller class.
 *
 */
class APPMAKER_WC_REST_Cart_Controller extends APPMAKER_WC_REST_Controller {

	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	protected $namespace = 'appmaker-wc/v1';

	/**
	 * Route base.
	 *
	 * @var string
	 */
	protected $rest_base = 'cart';


	/**
	 * Register the routes for products.
	 */
	public function register_routes() {
		register_rest_route( $this->namespace, '/' . $this->rest_base, array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_cart_items' ),
				'permission_callback' => array( $this, 'api_permissions_check' ),
			),
			'schema' => array( $this, 'get_public_item_schema' ),
		) );

		register_rest_route( $this->namespace, '/' . $this->rest_base . '/meta', array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_cart_meta' ),
				'permission_callback' => array( $this, 'api_permissions_check' ),
			),
			'schema' => array( $this, 'get_public_item_schema' ),
		) );

		register_rest_route( $this->namespace, '/' . $this->rest_base . '/add', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'add_to_cart' ),
				'permission_callback' => array( $this, 'api_permissions_check' ),
			),
			'schema' => array( $this, 'get_public_item_schema' ),
		) );

		register_rest_route( $this->namespace, '/' . $this->rest_base . '/set_quantity', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'set_quantity' ),
				'permission_callback' => array( $this, 'api_permissions_check' ),
			),
			'schema' => array( $this, 'get_public_item_schema' ),
		) );

		register_rest_route( $this->namespace, '/' . $this->rest_base . '/coupon/add', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'add_coupon' ),
				'permission_callback' => array( $this, 'api_permissions_check' ),
			),
			'schema' => array( $this, 'get_public_item_schema' ),
		) );

		register_rest_route( $this->namespace, '/' . $this->rest_base . '/coupon/remove', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'remove_coupon' ),
				'permission_callback' => array( $this, 'api_permissions_check' ),
			),
			'schema' => array( $this, 'get_public_item_schema' ),
		) );

	}

	/**
	 * Add to cart route.
	 *
	 * @param array $request Request data array.
	 *
	 * @return array|bool|WP_Error
	 */
	public function add_to_cart( $request ) {
		add_filter( 'option_woocommerce_cart_redirect_after_add', '__return_false', 9999 );
		add_filter( 'woocommerce_add_to_cart_redirect', '__return_false', 9999 );
		do_action( 'appmaker_wc_before_add_to_cart', $request );
		$return = apply_filters( 'appmaker_wc_add_to_cart_validate', array(), $request );
		$_POST['add-to-cart'] = $_REQUEST['add-to-cart'] = $request['product_id'];

		if ( isset( $request['quantity'] ) ) {
			$_POST['quantity'] = $_REQUEST['quantity'] = $request['quantity'];
		}

		if ( is_wp_error( $return ) ) {
			return $return;
		}

		if ( empty( $request['app-add-variations-to-cart'] ) ) {
			WC_Form_Handler::add_to_cart_action( false );
		}

		$return = $this->get_wc_notices_errors();

		if ( false !== $return ) {
			if ( ! is_wp_error( $return ) ) {
				$return = new WP_Error( 'error_add', __( 'Cannot add item to cart', 'appmaker_app' ) );
			}
		} else {
			$return = $this->cart_items();
		}

		return apply_filters( 'appmaker_wc_add_to_cart_response', $return );
	}

	public function get_cart_items(){
		if ( is_user_logged_in() ) {
			$cart = WC()->session->get( 'cart', null );
			if ( is_null( $cart ) || empty( $cart ) ) {
				WC()->cart->empty_cart( false );
				WC()->session->set( 'cart', null );
				WC()->cart->get_cart_from_session();
			}
		}
		return $this->cart_items();
	}

	/**
	 * Return cart items.
	 *
	 * @return array
	 */
	public function cart_items() {
		WC()->cart->calculate_totals();
		$cart               = WC()->cart->get_cart();
		$return             = $this->get_cart_meta();
		$return['products'] = array();
		foreach ( $cart as $key => $item ) {
			$item['key']      = $key;
			$variation        = array();
			$variation_string = '';
			if ( isset( $item['variation'] ) && is_array( $item['variation'] ) ) {
				foreach ( $item['variation'] as $id => $variation_value ) {
					// If this is a term slug, get the term's nice name.
					if ( taxonomy_exists( esc_attr( str_replace( 'attribute_', '', $id ) ) ) ) {
						$term = get_term_by( 'slug', $variation_value, esc_attr( str_replace( 'attribute_', '', $id ) ) );
						if ( ! is_wp_error( $term ) && ! empty( $term->name ) ) {
							$value = $term->name;
						} else {
							$value = ucwords( str_replace( '-', ' ', $variation_value ) );
						}
					} else {
						$value = ucwords( str_replace( '-', ' ', $variation_value ) );
					}
					$value       = $this->decode_html( trim( esc_html( apply_filters( 'woocommerce_variation_option_name', $value ) ) ), ENT_QUOTES, 'UTF-8' );
					$name        = $this->decode_html( wc_attribute_label( str_replace( 'attribute_', '', $id ) ) );
					$variation[] = array(
						'id'       => str_replace( 'attribute_', '', $id ),
						'name'     => $name,
						'value_id' => $variation_value,
						'value'    => $value,
					);
					$variation_string .= $name . ' : ' . $value . ' ';
				}
			}
			$item['variation']        = $variation;
			$item['variation_string'] = $variation_string;
			$item                     = array_merge( $item, $this->get_product_short_details( $item['data'] ) );
			unset( $item['data'] );
			$item['line_total_display'] = APPMAKER_WC_Helper::get_display_price( $item['line_total'] );
			$return['products'][]       = $item;
		}
		wc_clear_notices();
		WC()->cart->maybe_set_cart_cookies();

		return $return;
	}

	/**
	 * Rteurn cart meta.
	 *
	 * @return mixed|void
	 */
	public function get_cart_meta() {
		WC()->cart->calculate_totals();
		$coupon_discount_amounts = array();

		foreach ( WC()->cart->coupon_discount_amounts as $coupon => $price ) {
			$coupon_discount_amounts[] = array(
				'coupon'           => $coupon,
				'discount'         => $price,
				'discount_display' => APPMAKER_WC_Helper::get_display_price( $price )
			);
		}

		$return = array(
			'coupons_applied'       => WC()->cart->get_applied_coupons(),
			'coupon_discounted'     => $coupon_discount_amounts,
			'show_apply_coupon'     => apply_filters( 'woocommerce_coupons_enabled', 'yes' === get_option( 'woocommerce_enable_coupons' ) ),
			'enable_guest_checkout' => ( ( get_option( 'woocommerce_enable_guest_checkout' ) === 'yes' ) || ( get_option( 'woocommerce_enable_signup_and_login_from_checkout' ) === 'yes' ) ),
			'count'                 => WC()->cart->get_cart_contents_count(),
			'shipping_fee'          => ! empty( WC()->cart->shipping_total ) ? WC()->cart->shipping_total : 0,
			'shipping_fee_display'  => APPMAKER_WC_Helper::get_display_price( ! empty( WC()->cart->shipping_total ) ? WC()->cart->shipping_total : 0 ),
			'tax'                   => WC()->cart->get_cart_tax(),
			'fees'                  => WC()->cart->get_fees(),
			'currency'              => get_woocommerce_currency(),
			'currency_symbol'       => get_woocommerce_currency_symbol(),

			'total'       => WC()->cart->subtotal,
			'cart_total'  => WC()->cart->cart_contents_total,
			'order_total' => WC()->cart->total,

			'total_display'       => APPMAKER_WC_Helper::get_display_price( WC()->cart->subtotal ),
			'cart_total_display'  => APPMAKER_WC_Helper::get_display_price( WC()->cart->cart_contents_total ),
			'order_total_display' => APPMAKER_WC_Helper::get_display_price( WC()->cart->total ),
			'header_message'      => false,
			'price_format'        => get_woocommerce_price_format(),
			'timezone'            => wc_timezone_string(),
			'tax_included'        => ( 'yes' === get_option( 'woocommerce_prices_include_tax' ) ),
			'weight_unit'         => get_option( 'woocommerce_weight_unit' ),
			'dimension_unit'      => get_option( 'woocommerce_dimension_unit' ),
			'can_proceed'         => ! WC()->cart->is_empty(),
			'error_message'       => __( 'Cart is empty', 'appmaker-woocommerce-mobile-app-manager' ),
		);

		return apply_filters( 'appmaker_wc_cart_meta_response', $return );
	}

	/**
	 * Return product short detail.
	 *
	 * @param WC_Product $product Product Object.
	 *
	 * @return mixed
	 */
	public function get_product_short_details( $product ) {
		if ( ! is_a( $product, 'WC_Product' ) ) {
			$product = new WC_Product( $product );
		}
		$details['product_id'] = APPMAKER_WC_Helper::get_id( $product );
		// $details['product_total_stock'] = (int) $product->total_stock;
		$details['product_title'] = $product->get_title();
		$details['featured_src']  = wp_get_attachment_image_src( get_post_thumbnail_id( APPMAKER_WC_Helper::get_id( $product ) ) );

		$parent = method_exists( $product, 'get_parent_id' ) ? $product->get_parent_id() : ( ! empty( $product->parent ) ? $product->parent->id : false );

		if ( ! ( isset( $details['featured_src'][0] ) && ! empty( $details['featured_src'][0] ) ) && ! empty ( $parent ) ) {
			$details['featured_src'] = wp_get_attachment_image_src( get_post_thumbnail_id( $parent ) );
		}

		if ( isset( $details['featured_src'][0] ) && ! empty( $details['featured_src'][0] ) ) {
			$details['featured_src'] = $this->ensure_absolute_link( $details['featured_src'][0] );
		} else {
			$details['featured_src'] = $this->ensure_absolute_link( wc_placeholder_img_src() );
		}
		$details['product_type'] = $product->get_type();

		$details['product_price']         = $product->get_price();
		$details['product_regular_price'] = $product->get_regular_price();
		$details['product_sale_price']    = $product->get_sale_price();

		$details['product_price_display']         = APPMAKER_WC_Helper::get_display_price( $product->get_price() );
		$details['product_regular_price_display'] = APPMAKER_WC_Helper::get_display_price( $product->get_regular_price() );
		$details['product_sale_price_display']    = APPMAKER_WC_Helper::get_display_price( $product->get_sale_price() );

		$details['qty_config'] = $this->get_qty_args( $product, array( 'min_value' => 1 ) );

		return $details;
	}

	/**
	 * Return qty config.
	 *
	 * @param WC_Product $product Product object.
	 * @param array $args Arguments.
	 *
	 * @return array|mixed|void
	 */
	public function get_qty_args( $product, $args = array() ) {
		$defaults = array(
			'type'  => 'normal',
			'label'       => __( 'Quantity', 'woocommerce' ),
			'display'     => ! $product->is_sold_individually(),
			'input_value' => '1',
			'max_value'   => apply_filters( 'woocommerce_quantity_input_max', $product->backorders_allowed() ? '' : $product->get_stock_quantity(), $product ),
			'min_value'   => apply_filters( 'woocommerce_quantity_input_min', '1', $product ),
			'step'        => apply_filters( 'woocommerce_quantity_input_step', '1', $product ),
		);

		$args = apply_filters( 'woocommerce_quantity_input_args', wp_parse_args( $args, $defaults ), $product );

		// Set min and max value to empty string if not set.
		$args['min_value'] = isset( $args['min_value'] ) ? $args['min_value'] : '';
		$args['max_value'] = isset( $args['max_value'] ) ? $args['max_value'] : '';

		// Apply sanity to min/max args - min cannot be lower than 0.
		if ( '' !== $args['min_value'] && is_numeric( $args['min_value'] ) && $args['min_value'] < 0 ) {
			$args['min_value'] = 0; // Cannot be lower than 0.
		}

		// Max cannot be lower than 0 or min.
		if ( '' !== $args['max_value'] && is_numeric( $args['max_value'] ) ) {
			$args['max_value'] = $args['max_value'] < 0 ? 0 : $args['max_value'];
			$args['max_value'] = $args['max_value'] < $args['min_value'] ? $args['min_value'] : $args['max_value'];
		}

		if ( floatval( $args['input_value'] ) < floatval( $args['min_value'] ) ) {
			$args['input_value'] = $args['min_value'];
		}

		return $args;
	}

	/**
	 * Set quantity route.
	 *
	 * @param array $request Request data array.
	 *
	 * @return array|bool|WP_Error
	 */
	public function set_quantity( $request ) {
		$product = WC()->cart->get_cart_item( $request['key'] );
		if ( $request['quantity'] > 0 && ( isset( $product['data'] ) && ! $product['data']->has_enough_stock( $product['quantity'] + $request['quantity'] ) ) ) {
			return new WP_Error( 'cannot_add', sprintf( __( 'You cannot add that amount to the cart &mdash; we have %s in stock and you already have %s in your cart.', 'woocommerce' ), $product['data']->get_stock_quantity(), $product['quantity'] ) );
		} elseif ( isset( $product['data'] ) ) {
			WC()->cart->set_quantity( $request['key'], $request['quantity'], $request['refresh_totals'] );
		}

		WC()->cart->check_cart_items();
		WC()->cart->persistent_cart_update();

		return $this->cart_items();
	}

	/**
	 * Add coupon route.
	 *
	 * @param array $request Request data array.
	 *
	 * @return array|bool|WP_Error
	 */
	public function add_coupon( $request ) {
		$added = WC()->cart->add_discount( $request['coupon'] );
		if ( ! $added ) {
			$return = $this->get_wc_notices_errors();
			if ( ! is_wp_error( $return ) ) {
				$return = new WP_Error( 'invalid_coupon', 'Invalid coupon' );
			}
		} else {
			WC()->cart->persistent_cart_update();
			$return = $this->cart_items();
		}

		return $return;
	}

	/**
	 * Remove coupon route.
	 *
	 * @param array $request Request data array.
	 *
	 * @return array|bool|WP_Error
	 */
	public function remove_coupon( $request ) {
		$added = WC()->cart->remove_coupon( $request['coupon'] );
		if ( ! $added ) {
			$return = $this->get_wc_notices_errors();
			if ( ! is_wp_error( $return ) ) {
				$return = new WP_Error( 'invalid_coupon', 'Invalid coupon' );
			}
		} else {
			WC()->cart->persistent_cart_update();
			$return = $this->cart_items();
		}

		return $return;
	}



}
